From: Greg Kroah-Hartman <gregkh@linuxfoundation.org>
To: linux-kernel@vger.kernel.org
Cc: Greg Kroah-Hartman <gregkh@linuxfoundation.org>,
stable@vger.kernel.org, Theodore Tso <tytso@mit.edu>,
Dominik Brodowski <linux@dominikbrodowski.net>,
Eric Biggers <ebiggers@google.com>,
"Jason A. Donenfeld" <Jason@zx2c4.com>
Subject: [PATCH 5.17 040/111] random: unify early init crng load accounting
Date: Fri, 27 May 2022 10:49:12 +0200 [thread overview]
Message-ID: <20220527084825.140968745@linuxfoundation.org> (raw)
In-Reply-To: <20220527084819.133490171@linuxfoundation.org>
From: "Jason A. Donenfeld" <Jason@zx2c4.com>
commit da792c6d5f59a76c10a310c5d4c93428fd18f996 upstream.
crng_fast_load() and crng_slow_load() have different semantics:
- crng_fast_load() xors and accounts with crng_init_cnt.
- crng_slow_load() hashes and doesn't account.
However add_hwgenerator_randomness() can afford to hash (it's called
from a kthread), and it should account. Additionally, ones that can
afford to hash don't need to take a trylock but can take a normal lock.
So, we combine these into one function, crng_pre_init_inject(), which
allows us to control these in a uniform way. This will make it simpler
later to simplify this all down when the time comes for that.
Cc: Theodore Ts'o <tytso@mit.edu>
Reviewed-by: Dominik Brodowski <linux@dominikbrodowski.net>
Reviewed-by: Eric Biggers <ebiggers@google.com>
Signed-off-by: Jason A. Donenfeld <Jason@zx2c4.com>
Signed-off-by: Greg Kroah-Hartman <gregkh@linuxfoundation.org>
---
drivers/char/random.c | 114 +++++++++++++++++++++++++-------------------------
1 file changed, 59 insertions(+), 55 deletions(-)
--- a/drivers/char/random.c
+++ b/drivers/char/random.c
@@ -386,7 +386,7 @@ static void crng_make_state(u32 chacha_s
* For the fast path, we check whether we're ready, unlocked first, and
* then re-check once locked later. In the case where we're really not
* ready, we do fast key erasure with the base_crng directly, because
* this is what crng_{fast,slow}_load mutate during early init.
*/
if (unlikely(!crng_ready())) {
bool ready;
@@ -437,72 +437,75 @@ static void crng_make_state(u32 chacha_s
}
/*
- * This function is for crng_init == 0 only.
+ * This function is for crng_init == 0 only. It loads entropy directly
+ * into the crng's key, without going through the input pool. It is,
+ * generally speaking, not very safe, but we use this only at early
+ * boot time when it's better to have something there rather than
+ * nothing.
+ *
+ * There are two paths, a slow one and a fast one. The slow one
+ * hashes the input along with the current key. The fast one simply
+ * xors it in, and should only be used from interrupt context.
+ *
+ * If account is set, then the crng_init_cnt counter is incremented.
+ * This shouldn't be set by functions like add_device_randomness(),
+ * where we can't trust the buffer passed to it is guaranteed to be
+ * unpredictable (so it might not have any entropy at all).
*
- * crng_fast_load() can be called by code in the interrupt service
- * path. So we can't afford to dilly-dally. Returns the number of
- * bytes processed from cp.
+ * Returns the number of bytes processed from input, which is bounded
+ * by CRNG_INIT_CNT_THRESH if account is true.
*/
-static size_t crng_fast_load(const void *cp, size_t len)
+static size_t crng_pre_init_inject(const void *input, size_t len,
+ bool fast, bool account)
{
static int crng_init_cnt = 0;
unsigned long flags;
- const u8 *src = (const u8 *)cp;
- size_t ret = 0;
- if (!spin_trylock_irqsave(&base_crng.lock, flags))
- return 0;
+ if (fast) {
+ if (!spin_trylock_irqsave(&base_crng.lock, flags))
+ return 0;
+ } else {
+ spin_lock_irqsave(&base_crng.lock, flags);
+ }
+
if (crng_init != 0) {
spin_unlock_irqrestore(&base_crng.lock, flags);
return 0;
}
- while (len > 0 && crng_init_cnt < CRNG_INIT_CNT_THRESH) {
- base_crng.key[crng_init_cnt % sizeof(base_crng.key)] ^= *src;
- src++; crng_init_cnt++; len--; ret++;
- }
- if (crng_init_cnt >= CRNG_INIT_CNT_THRESH) {
- ++base_crng.generation;
- crng_init = 1;
- }
- spin_unlock_irqrestore(&base_crng.lock, flags);
- if (crng_init == 1)
- pr_notice("fast init done\n");
- return ret;
-}
-/*
- * This function is for crng_init == 0 only.
- *
- * crng_slow_load() is called by add_device_randomness, which has two
- * attributes. (1) We can't trust the buffer passed to it is
- * guaranteed to be unpredictable (so it might not have any entropy at
- * all), and (2) it doesn't have the performance constraints of
- * crng_fast_load().
- *
- * So, we simply hash the contents in with the current key. Finally,
- * we do *not* advance crng_init_cnt since buffer we may get may be
- * something like a fixed DMI table (for example), which might very
- * well be unique to the machine, but is otherwise unvarying.
- */
-static void crng_slow_load(const void *cp, size_t len)
-{
- unsigned long flags;
- struct blake2s_state hash;
+ if (account)
+ len = min_t(size_t, len, CRNG_INIT_CNT_THRESH - crng_init_cnt);
- blake2s_init(&hash, sizeof(base_crng.key));
-
- if (!spin_trylock_irqsave(&base_crng.lock, flags))
- return;
- if (crng_init != 0) {
- spin_unlock_irqrestore(&base_crng.lock, flags);
- return;
+ if (fast) {
+ const u8 *src = input;
+ size_t i;
+
+ for (i = 0; i < len; ++i)
+ base_crng.key[(crng_init_cnt + i) %
+ sizeof(base_crng.key)] ^= src[i];
+ } else {
+ struct blake2s_state hash;
+
+ blake2s_init(&hash, sizeof(base_crng.key));
+ blake2s_update(&hash, base_crng.key, sizeof(base_crng.key));
+ blake2s_update(&hash, input, len);
+ blake2s_final(&hash, base_crng.key);
+ }
+
+ if (account) {
+ crng_init_cnt += len;
+ if (crng_init_cnt >= CRNG_INIT_CNT_THRESH) {
+ ++base_crng.generation;
+ crng_init = 1;
+ }
}
- blake2s_update(&hash, base_crng.key, sizeof(base_crng.key));
- blake2s_update(&hash, cp, len);
- blake2s_final(&hash, base_crng.key);
-
spin_unlock_irqrestore(&base_crng.lock, flags);
+
+ if (crng_init == 1)
+ pr_notice("fast init done\n");
+
+ return len;
}
static void _get_random_bytes(void *buf, size_t nbytes)
@@ -1018,7 +1021,7 @@ void add_device_randomness(const void *b
unsigned long flags;
if (!crng_ready() && size)
- crng_slow_load(buf, size);
+ crng_pre_init_inject(buf, size, false, false);
spin_lock_irqsave(&input_pool.lock, flags);
_mix_pool_bytes(buf, size);
@@ -1135,7 +1138,7 @@ void add_hwgenerator_randomness(const vo
size_t entropy)
{
if (unlikely(crng_init == 0)) {
- size_t ret = crng_fast_load(buffer, count);
+ size_t ret = crng_pre_init_inject(buffer, count, false, true);
mix_pool_bytes(buffer, ret);
count -= ret;
buffer += ret;
@@ -1298,7 +1301,8 @@ void add_interrupt_randomness(int irq)
if (unlikely(crng_init == 0)) {
if (new_count >= 64 &&
- crng_fast_load(fast_pool->pool32, sizeof(fast_pool->pool32)) > 0) {
+ crng_pre_init_inject(fast_pool->pool32, sizeof(fast_pool->pool32),
+ true, true) > 0) {
atomic_set(&fast_pool->count, 0);
fast_pool->last = now;
if (spin_trylock(&input_pool.lock)) {
next prev parent reply other threads:[~2022-05-27 11:40 UTC|newest]
Thread overview: 118+ messages / expand[flat|nested] mbox.gz Atom feed top
2022-05-27 8:48 [PATCH 5.17 000/111] 5.17.12-rc1 review Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 001/111] HID: amd_sfh: Add support for sensor discovery Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 002/111] KVM: x86/mmu: fix NULL pointer dereference on guest INVPCID Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 003/111] random: use computational hash for entropy extraction Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 004/111] random: simplify entropy debiting Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 005/111] random: use linear min-entropy accumulation crediting Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 006/111] random: always wake up entropy writers after extraction Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 007/111] random: make credit_entropy_bits() always safe Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 008/111] random: remove use_input_pool parameter from crng_reseed() Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 009/111] random: remove batched entropy locking Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 010/111] random: fix locking in crng_fast_load() Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 011/111] random: use RDSEED instead of RDRAND in entropy extraction Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 012/111] random: get rid of secondary crngs Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 013/111] random: inline leaves of rand_initialize() Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 014/111] random: ensure early RDSEED goes through mixer on init Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 015/111] random: do not xor RDRAND when writing into /dev/random Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 016/111] random: absorb fast pool into input pool after fast load Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 017/111] random: use simpler fast key erasure flow on per-cpu keys Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 018/111] random: use hash function for crng_slow_load() Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 019/111] random: make more consistent use of integer types Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 020/111] random: remove outdated INT_MAX >> 6 check in urandom_read() Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 021/111] random: zero buffer after reading entropy from userspace Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 022/111] random: fix locking for crng_init in crng_reseed() Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 023/111] random: tie batched entropy generation to base_crng generation Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 024/111] random: remove ifdefd out interrupt bench Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 025/111] random: remove unused tracepoints Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 026/111] random: add proper SPDX header Greg Kroah-Hartman
2022-05-27 8:48 ` [PATCH 5.17 027/111] random: deobfuscate irq u32/u64 contributions Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 028/111] random: introduce drain_entropy() helper to declutter crng_reseed() Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 029/111] random: remove useless header comment Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 030/111] random: remove whitespace and reorder includes Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 031/111] random: group initialization wait functions Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 032/111] random: group crng functions Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 033/111] random: group entropy extraction functions Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 034/111] random: group entropy collection functions Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 035/111] random: group userspace read/write functions Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 036/111] random: group sysctl functions Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 037/111] random: rewrite header introductory comment Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 038/111] random: defer fast pool mixing to worker Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 039/111] random: do not take pool spinlock at boot Greg Kroah-Hartman
2022-05-27 8:49 ` Greg Kroah-Hartman [this message]
2022-05-27 8:49 ` [PATCH 5.17 041/111] random: check for crng_init == 0 in add_device_randomness() Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 042/111] random: pull add_hwgenerator_randomness() declaration into random.h Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 043/111] random: clear fast pool, crng, and batches in cpuhp bring up Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 044/111] random: round-robin registers as ulong, not u32 Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 045/111] random: only wake up writers after zap if threshold was passed Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 046/111] random: cleanup UUID handling Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 047/111] random: unify cycles_t and jiffies usage and types Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 048/111] random: do crng pre-init loading in worker rather than irq Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 049/111] random: give sysctl_random_min_urandom_seed a more sensible value Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 050/111] random: dont let 644 read-only sysctls be written to Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 051/111] random: replace custom notifier chain with standard one Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 052/111] random: use SipHash as interrupt entropy accumulator Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 053/111] random: make consistent usage of crng_ready() Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 054/111] random: reseed more often immediately after booting Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 055/111] random: check for signal and try earlier when generating entropy Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 056/111] random: skip fast_init if hwrng provides large chunk of entropy Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 057/111] random: treat bootloader trust toggle the same way as cpu trust toggle Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 058/111] random: re-add removed comment about get_random_{u32,u64} reseeding Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 059/111] random: mix build-time latent entropy into pool at init Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 060/111] random: do not split fast init input in add_hwgenerator_randomness() Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 061/111] random: do not allow user to keep crng key around on stack Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 062/111] random: check for signal_pending() outside of need_resched() check Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 063/111] random: check for signals every PAGE_SIZE chunk of /dev/[u]random Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 064/111] random: allow partial reads if later user copies fail Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 065/111] random: make random_get_entropy() return an unsigned long Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 066/111] random: document crng_fast_key_erasure() destination possibility Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 067/111] random: fix sysctl documentation nits Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 068/111] init: call time_init() before rand_initialize() Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 069/111] ia64: define get_cycles macro for arch-override Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 070/111] s390: " Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 071/111] parisc: " Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 072/111] alpha: " Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 073/111] powerpc: " Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 074/111] timekeeping: Add raw clock fallback for random_get_entropy() Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 075/111] m68k: use fallback for random_get_entropy() instead of zero Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 076/111] riscv: " Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 077/111] mips: use fallback for random_get_entropy() instead of just c0 random Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 078/111] arm: use fallback for random_get_entropy() instead of zero Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 079/111] nios2: " Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 080/111] x86/tsc: Use " Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 081/111] um: use " Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 082/111] sparc: " Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 083/111] xtensa: " Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 084/111] random: insist on random_get_entropy() existing in order to simplify Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 085/111] random: do not use batches when !crng_ready() Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 086/111] random: use first 128 bits of input as fast init Greg Kroah-Hartman
2022-05-27 8:49 ` [PATCH 5.17 087/111] random: do not pretend to handle premature next security model Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 088/111] random: order timer entropy functions below interrupt functions Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 089/111] random: do not use input pool from hard IRQs Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 090/111] random: help compiler out with fast_mix() by using simpler arguments Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 091/111] siphash: use one source of truth for siphash permutations Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 092/111] random: use symbolic constants for crng_init states Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 093/111] random: avoid initializing twice in credit race Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 094/111] random: move initialization out of reseeding hot path Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 095/111] random: remove ratelimiting for in-kernel unseeded randomness Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 096/111] random: use proper jiffies comparison macro Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 097/111] random: handle latent entropy and command line from random_init() Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 098/111] random: credit architectural init the exact amount Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 099/111] random: use static branch for crng_ready() Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 100/111] random: remove extern from functions in header Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 101/111] random: use proper return types on get_random_{int,long}_wait() Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 102/111] random: make consistent use of buf and len Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 103/111] random: move initialization functions out of hot pages Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 104/111] random: move randomize_page() into mm where it belongs Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 105/111] random: unify batched entropy implementations Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 106/111] random: convert to using fops->read_iter() Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 107/111] random: convert to using fops->write_iter() Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 108/111] random: wire up fops->splice_{read,write}_iter() Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 109/111] random: check for signals after page of pool writes Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 110/111] ACPI: sysfs: Fix BERT error region memory mapping Greg Kroah-Hartman
2022-05-27 8:50 ` [PATCH 5.17 111/111] ALSA: ctxfi: Add SB046x PCI ID Greg Kroah-Hartman
2022-05-27 22:39 ` [PATCH 5.17 000/111] 5.17.12-rc1 review Guenter Roeck
2022-05-28 2:00 ` Justin Forbes
2022-05-28 10:43 ` Naresh Kamboju
2022-05-28 11:06 ` Ron Economos
2022-05-28 15:01 ` Fox Chen
2022-05-28 15:25 ` Sudip Mukherjee
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20220527084825.140968745@linuxfoundation.org \
--to=gregkh@linuxfoundation.org \
--cc=Jason@zx2c4.com \
--cc=ebiggers@google.com \
--cc=linux-kernel@vger.kernel.org \
--cc=linux@dominikbrodowski.net \
--cc=stable@vger.kernel.org \
--cc=tytso@mit.edu \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -386,7 +386,7 @@ static void crng_make_state(u32 chacha_s
* then re-check once locked later. In the case where we're really not
* ready, we do fast key erasure with the base_crng directly, because
- * this is what crng_{fast,slow}_load mutate during early init.
+ * this is what crng_pre_init_inject() mutates during early init.
*/
if (unlikely(!crng_ready())) {
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox